@@ -93,8 +93,8 @@ import Prelude hiding (map)
 import qualified Data.HashMap.Array as A
 import qualified Data.HashMap.Base as HM
 import Data.HashMap.Base hiding (
-    adjust, fromList, fromListWith, insert, insertWith, intersectionWith, map,
-    singleton, unionWith)
+    adjust, fromList, fromListWith, insert, insertWith, intersectionWith,
+    lookupDefault, map, singleton, unionWith)
 import Data.HashMap.Unsafe (runST)
 
 ------------------------------------------------------------------------
@@ -107,6 +107,14 @@ singleton k !v = HM.singleton k v
 ------------------------------------------------------------------------
 -- * Basic interface
 
+-- | /O(log n)/ Return the value to which the specified key is mapped,
+-- or the default value if this map contains no mapping for the key.
+lookupDefault :: (Eq k, Hashable k)
+              => v          -- ^ Default value to return.
+              -> k -> HashMap k v -> v
+lookupDefault !def k t = HM.lookupDefault def k t
+{-# INLINABLE lookupDefault #-}
+
 -- | /O(log n)/ Associate the specified value with the specified
 -- key in this map.  If this map previously contained a mapping for
 -- the key, the old value is replaced.
@@ -318,15 +326,19 @@ unionWith f = go 0
 -- * Transformations
 
 -- | /O(n)/ Transform this map by applying a function to every value.
-map :: (v1 -> v2) -> HashMap k v1 -> HashMap k v2
-map f = go
+mapWithKey :: (k -> v1 -> v2) -> HashMap k v1 -> HashMap k v2
+mapWithKey f = go
   where
     go Empty                 = Empty
-    go (Leaf h (L k v))      = let !v' = f v in Leaf h $ L k v'
+    go (Leaf h (L k v))      = let !v' = f k v in Leaf h $ L k v'
     go (BitmapIndexed b ary) = BitmapIndexed b $ A.map' go ary
     go (Full ary)            = Full $ A.map' go ary
     go (Collision h ary)     =
-        Collision h $ A.map' (\ (L k v) -> let !v' = f v in L k v') ary
+        Collision h $ A.map' (\ (L k v) -> let !v' = f k v in L k v') ary
+{-# INLINE mapWithKey #-}
+
+map :: (v1 -> v2) -> HashMap k v1 -> HashMap k v2
+map f = mapWithKey (const f)
 {-# INLINE map #-}
 
 -- TODO: Should we add a strict traverseWithKey?
