@@ -1,4 +1,4 @@
- {-# LANGUAGE CPP, GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE CPP, GeneralizedNewtypeDeriving #-}
 
 -- | Tests for the 'Data.HashMap.Lazy' module.  We test functions by
 -- comparing them to a simpler model, an association list.
@@ -7,7 +7,7 @@ module Main (main) where
 
 import qualified Data.Foldable as Foldable
 import Data.Function (on)
-import Data.Hashable (Hashable(hash))
+import Data.Hashable (Hashable(hashWithSalt))
 import qualified Data.List as L
 #if defined(STRICT)
 import qualified Data.HashMap.Strict as HM
@@ -24,7 +24,7 @@ newtype Key = K { unK :: Int }
             deriving (Arbitrary, Eq, Ord, Show)
 
 instance Hashable Key where
-    hash k = hash (unK k) `mod` 20
+    hashWithSalt salt k = hashWithSalt salt (unK k) `mod` 20
 
 ------------------------------------------------------------------------
 -- * Properties
@@ -67,7 +67,7 @@ newtype AlwaysCollide = AC Int
     deriving (Arbitrary, Eq, Ord, Show)
 
 instance Hashable AlwaysCollide where
-    hash _ = 1
+    hashWithSalt _ _ = 1
 
 -- White-box test that tests the case of deleting one of two keys from
 -- a map, where the keys' hash values collide.
@@ -142,7 +142,15 @@ pFoldrWithKey = (sortByKey . M.foldrWithKey f []) `eq`
   where f k v z = (k, v) : z
 
 pFoldl' :: Int -> [(Int, Int)] -> Bool
-pFoldl' z0 = M.foldlWithKey' (\ z _ v -> v + z) z0 `eq` HM.foldl' (+) z0
+pFoldl' z0 = foldlWithKey'Map (\ z _ v -> v + z) z0 `eq` HM.foldl' (+) z0
+
+foldlWithKey'Map :: (b -> k -> a -> b) -> b -> M.Map k a -> b
+#if MIN_VERSION_containers(4,2,0)
+foldlWithKey'Map = M.foldlWithKey'
+#else
+-- Equivalent except for bottoms, which we don't test.
+foldlWithKey'Map = M.foldlWithKey
+#endif
 
 ------------------------------------------------------------------------
 -- ** Filter
